'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _d = require('d3');

var _d2 = _interopRequireDefault(_d);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var LineChart = (function () {
  function LineChart(config) {
    (0, _classCallCheck3.default)(this, LineChart);

    this.config = (0, _assign2.default)(config, LineChart.defaultConfig);

    // 实例唯一ID
    this.id = Math.random().toString(16).slice(2);
    this.series = this.getSeries();
    this.categories = this.getCategories();

    this.createContainer();
    this.createAxis();
    this.createGeometry();
    this.createTooltip();
    this.createLegend();
    this.createMarkers();
    this.startAnimation();
  }

  /**
   * 获取 data 与 cols 关联后的数据
   */

  (0, _createClass3.default)(LineChart, [{
    key: 'getSeries',
    value: function getSeries() {
      var _config = this.config;
      var cols = _config.cols;
      var data = _config.data;

      var series = (0, _keys2.default)(cols).map(function (key) {
        return {
          key: key,
          name: cols[key],
          data: []
        };
      });
      data.forEach(function (item) {
        series.forEach(function (_ref) {
          var key = _ref.key;
          var data = _ref.data;

          data.push(item[key]);
        });
      });
      return series;
    }

    /**
     * 获取 X 轴分类的数据
     */

  }, {
    key: 'getCategories',
    value: function getCategories() {
      var _config2 = this.config;
      var category = _config2.category;
      var data = _config2.data;

      return data.map(function (item) {
        return item[category];
      });
    }

    /**
     * 获取 Y 轴值域，最大值适当增大，预留边距
     */

  }, {
    key: 'getYDomain',
    value: function getYDomain(series) {
      var domain = _d2.default.extent(series.reduce(function (current, next) {
        [].push.apply(current, _d2.default.extent(next.data));
        return current;
      }, []));
      if (domain[0] > 0) {
        domain[0] = 0;
      }
      domain[1] *= 1.1;
      return domain;
    }

    /**
     * 计算 Y 轴 tickValues，实现无边距均分效果
     */

  }, {
    key: 'getYTickValues',
    value: function getYTickValues() {
      var max = this.yScale.invert(0);
      var values = [];
      var step = max / 5;
      for (var i = 0; i <= 5; i++) {
        values.push(step * i);
      }
      return values;
    }

    /**
     * 创建容器
     */

  }, {
    key: 'createContainer',
    value: function createContainer() {
      var container = this.config.container;

      var width = container.clientWidth;
      var height = container.clientHeight || width * .5;
      this.padding = [30, 10, 30, 40];

      var _padding = (0, _slicedToArray3.default)(this.padding, 4);

      var top = _padding[0];
      var right = _padding[1];
      var bottom = _padding[2];
      var left = _padding[3];

      this.container = _d2.default.select(container).html('').append('svg').attr('width', width).attr('height', height).append('g').attr('transform', 'translate(' + left + ', ' + top + ')');
      this.innerWidth = width - left - right;
      this.innerHeight = height - top - bottom;
    }

    // 获取 X 轴两边边距因子

  }, {
    key: 'getXAxisPaddingScale',
    value: function getXAxisPaddingScale() {
      return 20 / (this.innerWidth / this.categories.length);
    }
  }, {
    key: 'drawXAxis',
    value: function drawXAxis() {
      this.xScale.rangePoints([0, this.innerWidth], this.getXAxisPaddingScale());
      this.xAxis.scale(this.xScale);

      // 处理x轴数据节点过多的问题
      var interval = Math.ceil(this.categories.join('').length / (this.innerWidth / 10));
      this.xAxis.tickValues(this.xScale.domain().filter(function (d, i) {
        return !(i % interval);
      }));

      this.container.select('.axis-x').attr('transform', 'translate(0, ' + this.innerHeight + ')').call(this.xAxis);
    }

    /**
     * 绘制坐标轴
     */

  }, {
    key: 'createAxis',
    value: function createAxis() {
      var yAxis = this.config.yAxis;

      this.xScale = _d2.default.scale.ordinal().domain(this.categories);

      this.yScale = _d2.default.scale.linear().domain(this.getYDomain(this.series)).range([this.innerHeight, 0]);

      this.xAxis = _d2.default.svg.axis().orient('bottom').tickSize(0, 0).tickPadding(10);

      var format = _d2.default.format(yAxis.format || '.2s');
      this.yAxis = _d2.default.svg.axis().scale(this.yScale).orient('left').tickValues(this.getYTickValues()).tickFormat(function (d) {
        return format(d);
      }).tickSize(-this.innerWidth, 10);

      this.container.append('g').attr('class', 'axis-y').call(this.yAxis);

      this.container.append('g').attr('class', 'axis-x');
      this.drawXAxis();
    }

    /**
     * 绘制曲线、曲面
     */

  }, {
    key: 'createGeometry',
    value: function createGeometry() {
      var _this = this;

      var _config3 = this.config;
      var colors = _config3.colors;
      var data = _config3.data;
      var category = _config3.category;

      var seriesGroup = this.container.append('g').attr('class', 'series-group');

      // 弯曲类型
      var INTERPOLATE = 'cardinal';

      this.linePathGenerator = _d2.default.svg.line().interpolate(INTERPOLATE).x(function (d) {
        return _this.xScale(d[category]);
      });

      this.areaPathGenerator = _d2.default.svg.area().interpolate(INTERPOLATE).x(function (d) {
        return _this.xScale(d[category]);
      }).y0(this.innerHeight);

      this.series.forEach(function (_ref2, i) {
        var key = _ref2.key;

        var serieGroup = seriesGroup.append('g').attr('class', 'serie-group serie-group-' + key);
        var color = colors[i];

        // 曲线
        serieGroup.append('path').attr('class', 'line').datum(data).attr('d', _this.linePathGenerator.y(function (d) {
          return _this.yScale(d[key]);
        })).attr('fill', 'none').attr('stroke', color).attr('stroke-width', 2);

        // 曲面渐变效果
        var gradientID = 'gradient-' + _this.id + '-' + i;
        var linearGradient = serieGroup.append('linearGradient').attr('id', gradientID).attr('x1', '50%').attr('y1', '0%').attr('x2', '50%').attr('y2', '100%');
        linearGradient.append('stop').attr('offset', 0).style('stop-color', color);
        linearGradient.append('stop').attr('offset', 1).style('stop-color', '#fff');

        // 曲面
        serieGroup.append('path').attr('class', 'area').datum(data).attr('d', _this.areaPathGenerator.y1(function (d) {
          return _this.yScale(d[key]);
        })).style({
          fill: 'url(#' + gradientID + ')',
          opacity: 0.4
        });
      });
    }

    /**
     * tooltip功能：参考线、参考线与 x 轴交汇的圆点、详细数据浮层
     */

  }, {
    key: 'createTooltip',
    value: function createTooltip() {
      var container = this.config.container;

      var padding = this.getXAxisPaddingScale();

      // x轴边距实际大小
      this.xAxisPadding = this.innerWidth / (this.categories.length - 1 + padding) * padding / 2;

      // 存储最后的索引，index 未变化时不再重复绘制(参考线、tooltip等)
      this.lastxAxisIndex = null;

      // 参考线
      this.activeLine = this.container.append('line').attr('y2', 0).attr('y1', this.innerHeight).attr('class', 'active-line').style('opacity', 0);

      // 参考线与 x 轴交汇的圆点
      this.activeCircle = this.container.append('circle').style('opacity', 0).attr('class', 'active-circle').attr('r', 2).attr('cy', this.innerHeight);

      // tooltip 浮层对象节点
      this.tooltip = _d2.default.select(container).append('div').attr('class', 'bfd-line-chart__tooltip').style({
        position: 'absolute',
        left: 0,
        top: 0,
        opacity: 0,
        'pointer-events': 'none'
      });

      this.createTooltipTrigger();
    }

    /**
     * 创建 Tooltip 功能触发器
     */

  }, {
    key: 'createTooltipTrigger',
    value: function createTooltipTrigger() {
      var _this2 = this;

      // 动画时长（参考线、tooltip平滑过渡）
      var duration = 400 / this.categories.length;

      this.container.append('rect').attr('fill', 'none').attr('pointer-events', 'all').attr('width', this.innerWidth).attr('height', this.innerHeight).on('mouseover', function () {
        _this2.drawTooltip(0);
      }).on('mousemove', function () {
        _this2.drawTooltip(duration);
      }).on('mouseout', function () {
        _this2.lastxAxisIndex = null;
        _this2.activeLine.style('opacity', 0);
        _this2.activeCircle.style('opacity', 0);
        _this2.tooltip.transition().delay(200).style('opacity', 0);
      });
    }

    /**
     * 获取鼠标在 X 轴上的索引
     */

  }, {
    key: 'getMouseIndexInXAxis',
    value: function getMouseIndexInXAxis() {
      var xAxisIndex = (this.categories.length - 1) * (_d2.default.mouse(_d2.default.event.currentTarget)[0] - this.xAxisPadding) / (this.innerWidth - this.xAxisPadding * 2);
      return +xAxisIndex.toFixed(0) || 0;
    }

    /**
     * 绘制 Tooltip，包括浮层、参考线等
     */

  }, {
    key: 'drawTooltip',
    value: function drawTooltip(duration) {
      var yAxis = this.config.yAxis;

      var xAxisIndex = this.getMouseIndexInXAxis();
      if (this.lastxAxisIndex === xAxisIndex) return;

      this.lastxAxisIndex = xAxisIndex;

      // x 轴坐标
      var x = this.xScale(this.categories[xAxisIndex]);

      // 绘制参考线
      this.activeLine.transition().duration(duration).attr('x1', x).attr('x2', x).style('opacity', 1);

      // 绘制参考线与 x 轴交汇的圆点
      this.activeCircle.transition().duration(duration).attr('cx', x).style('opacity', 1);

      // Tooltip 浮层内容
      var format = _d2.default.format(yAxis.format || '.2s');
      var html = this.series.reduce(function (current, serie) {
        return current + ('\n        <tr>\n          <td>' + serie.name + ':</td>\n          <td>' + format(serie.data[xAxisIndex]) + '</td>\n        </tr>\n      ');
      }, '<h2>' + this.categories[xAxisIndex] + '</h2><table>') + '</table>';
      this.tooltip.html(html);

      // 绘制浮层
      var _tooltip$0$ = this.tooltip[0][0];
      var clientWidth = _tooltip$0$.clientWidth;
      var clientHeight = _tooltip$0$.clientHeight;

      var relativeSize = -(clientWidth / 2 + 15);
      if (x - 10 < clientWidth) {
        relativeSize = Math.abs(relativeSize);
      }

      this.tooltip.transition().duration(duration).style({
        left: x + relativeSize + 'px',
        top: _d2.default.event.offsetY - clientHeight / 2 + 'px',
        opacity: 1
      });
    }

    /**
     * 标记点。没有和线、面一起绘制的原因是因为需要放到最顶层，防止被 activeLine 覆盖
     */

  }, {
    key: 'createMarkers',
    value: function createMarkers() {
      var _this3 = this;

      var _config4 = this.config;
      var category = _config4.category;
      var colors = _config4.colors;
      var data = _config4.data;

      var markersGroup = this.container.append('g').attr('class', 'markers-group');

      this.series.forEach(function (_ref3, i) {
        var key = _ref3.key;

        var markerGroup = markersGroup.append('g').attr('class', 'marker-group-' + key);

        var gEnter = markerGroup.selectAll('g').data(data).enter().append('g').attr('class', 'marker').attr('transform', function (d) {
          return 'translate(' + _this3.xScale(d[category]) + ', ' + _this3.yScale(d[key]) + ')';
        });

        gEnter.append('circle').attr('class', 'marker-inner').attr('r', 5).attr('fill', '#fff').attr('stroke', '#fff');

        gEnter.append('circle').attr('class', 'marker-outer').attr('r', 3.5).attr('fill', '#fff').attr('stroke', colors[i]).attr('stroke-width', 2);
      });
    }

    /**
     * 图例
     */

  }, {
    key: 'createLegend',
    value: function createLegend() {
      var _this4 = this;

      var _config5 = this.config;
      var container = _config5.container;
      var category = _config5.category;
      var colors = _config5.colors;

      var disabledMap = {};
      var t = _d2.default.transition().duration(500);

      var selection = _d2.default.select(container).append('div').attr('class', 'bfd-line-chart__legend').selectAll('div').data(this.series).enter().append('div').style('cursor', 'pointer').on('click', function (_ref4) {
        var key = _ref4.key;

        var node = _d2.default.select(_d2.default.event.currentTarget);
        var disabled = !node.classed('disabled');
        disabledMap[key] = disabled;
        var series = _this4.series.filter(function (_ref5) {
          var key = _ref5.key;
          return !disabledMap[key];
        });

        // 至少保留一条系列
        if (disabled && !series.length) {
          disabledMap[key] = false;
          return;
        }
        node.classed('disabled', disabled);

        _this4.yScale.domain(_this4.getYDomain(series)).nice();
        _this4.yAxis.tickValues(_this4.getYTickValues());

        // svg 重绘
        _this4.container.select('.axis-y').transition(t).call(_this4.yAxis);

        var serieGroup = _this4.container.select('.serie-group-' + key);
        var markerGroup = _this4.container.select('.marker-group-' + key);
        var display = disabled ? 'none' : 'block';
        serieGroup.style('display', display);
        markerGroup.style('display', display);

        series.forEach(function (_ref6) {
          var key = _ref6.key;

          if (disabledMap[key]) return;

          var serieGroup = _this4.container.select('.serie-group-' + key).transition(t);
          serieGroup.select('path.line').attr('d', _this4.linePathGenerator.y(function (d) {
            return _this4.yScale(d[key]);
          }));
          serieGroup.select('path.area').attr('d', _this4.areaPathGenerator.y1(function (d) {
            return _this4.yScale(d[key]);
          }));

          var markerGroup = _this4.container.select('.marker-group-' + key).transition(t);
          markerGroup.selectAll('.marker').attr('transform', function (d) {
            return 'translate(' + _this4.xScale(d[category]) + ', ' + _this4.yScale(d[key]) + ')';
          });
        });
      });

      selection.each(function (d, i) {
        var node = _d2.default.select(selection[0][i]);
        node.append('span').attr('class', 'line').style('background-color', colors[i]);
        node.append('span').attr('class', 'circle').style('border-color', colors[i]);
        node.append('span').attr('class', 'text').text(d.name);
      });
    }

    /**
     * 初始化渲染动画
     */

  }, {
    key: 'startAnimation',
    value: function startAnimation() {

      // 动画元素矢量蒙板
      var clipId = 'rectClip-' + this.id;
      this.container.select('.series-group').attr('clip-path', 'url(#' + clipId + ')');
      this.container.select('.markers-group').attr('clip-path', 'url(#' + clipId + ')');

      this.container.append('clipPath').attr('id', clipId).append('rect').attr('width', 0).attr('height', this.innerHeight - 1).transition().duration(900).attr('width', this.innerWidth);
    }
  }]);
  return LineChart;
})(); /**
       * Copyright 2016-present, Baifendian, Inc.
       * All rights reserved.
       *
       * This source code is licensed under the BSD-style license found in the
       * LICENSE file in the root directory of this source tree. An additional grant
       * of patent rights can be found in the PATENTS file in the same directory.
       */

LineChart.defaultConfig = {
  yAxis: {},
  colors: ['#26c6da', '#5c6bc0', '#f9ce1d', '#9c27b0']
};
exports.default = LineChart;