'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

require('./main.css');

var _d = require('d3');

var _d2 = _interopRequireDefault(_d);

var _pie = require('./pie');

var _pie2 = _interopRequireDefault(_pie);

var _assembleTooltip = require('./assembleTooltip');

var _assembleTooltip2 = _interopRequireDefault(_assembleTooltip);

var _assembleLegend = require('./assembleLegend');

var _assembleLegend2 = _interopRequireDefault(_assembleLegend);

var _color = require('./color');

var _color2 = _interopRequireDefault(_color);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Copyright 2016-present, Baifendian, Inc.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree. An additional grant
 * of patent rights can be found in the PATENTS file in the same directory.
 */

var _class = function _class(config) {
    (0, _classCallCheck3.default)(this, _class);

    var padding = [100, 60];
    var container = config.container;
    var data = config.data;
    var name = config.name;
    var animation = config.animation;
    var radius = config.radius;
    var tooltip = config.tooltip;
    var legend = config.legend;

    var env = {};
    env.container = container;
    env.config = {};
    env.config.data = data;
    env.config.name = name;
    env.config.legend = {
        layout: legend.layout || 'vertical',
        align: legend.align || 'right',
        style: {
            x: legend.style.x || 0,
            y: legend.style.y || 0
        }
    };
    if (animation) env.config.animation = animation;
    if (radius) env.config.radius = radius;
    if (tooltip) env.config.tooltip = tooltip;

    env.container.style.position = 'relative';

    env.config.legend.layout == 'vertical' ? env.width = env.container.clientWidth - padding[0] : env.width = env.container.clientWidth;
    env.height = env.container.clientHeight || env.container.clientWidth;

    if (!env.config.data) return;

    // 获取颜色。
    var colors = config.colors || (0, _color2.default)(2, env.config.data.length);

    for (var i = 0; i < env.config.data.length; i++) {
        env.config.data[i].color = colors[i];
        env.config.data[i].id = i;
    }

    env.config.dataLegend = [];
    for (var k in env.config.data) {
        env.config.dataLegend.push(env.config.data[k]);
    }env.svg = _d2.default.select(env.container).append('svg').attr('width', env.width).attr('height', env.height).append('g');

    if (env.config.legend.align == 'left') _d2.default.select(env.container).select('svg').style('margin-left', padding[0] + 'px');
    if (env.config.legend.align == 'top') _d2.default.select(env.container).select('svg').style('margin-top', padding[1] + 'px');
    if (env.config.legend.align == 'bottom') _d2.default.select(env.container).select('svg').style('margin-bottom', padding[1] + 'px');

    env.svg.attr('transform', 'translate(' + (env.width / 2 + 10) + ',' + (env.width / 2 + 10) + ')');

    // 添加3个g标签，分别是 pie-lines pie-slices pie-labels
    env.svg.append('g').attr('class', 'pie-lines');
    env.svg.append('g').attr('class', 'pie-slices');
    env.svg.append('g').attr('class', 'pie-labels');

    /*
     *init legend
     */
    (0, _assembleLegend2.default)(env);

    /*
     *init pie
     */
    (0, _pie2.default)(env, true);

    /*
     *init tooltip
     */
    if (!env.config.tooltip || env.config.tooltip.enabled !== false) {
        (0, _assembleTooltip2.default)(env, true);
    }

    /*
     *自适应容器大小
     */

    var __w = env.container.clientWidth;
    window.onresize = function () {

        if (env.container.clientWidth != __w) {

            env.svg.select('.pie-slices').selectAll('path').remove();
            env.svg.select('.pie-labels').selectAll('text').remove();
            env.svg.select('.pie-lines').selectAll('polyline').remove();

            env.config.legend.layout == 'vertical' ? env.width = env.container.clientWidth - padding[0] : env.width = env.container.clientWidth;
            env.svg.attr('transform', 'translate(' + (env.width / 2 + 10) + ',' + (env.width / 2 + 10) + ')');

            _d2.default.select(env.container).select('svg').attr('width', env.width).attr('height', env.height);

            (0, _pie2.default)(env, false);

            (0, _assembleTooltip2.default)(env, false);

            __w = env.container.clientWidth;
        }
    };
};

exports.default = _class;