'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _d = require('d3');

var _d2 = _interopRequireDefault(_d);

var _svg = require('../Svg/svg');

var _svg2 = _interopRequireDefault(_svg);

var _stackedRect = require('../Svg/stackedRect');

var _stackedRect2 = _interopRequireDefault(_stackedRect);

var _legend = require('../Svg/legend');

var _legend2 = _interopRequireDefault(_legend);

var _color = require('../Svg/color');

var _color2 = _interopRequireDefault(_color);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _class = (function () {
  (0, _createClass3.default)(_class, [{
    key: 'getDomain',

    /**
     * 获取Y轴最大最小值区间
     */
    value: function getDomain(series) {
      var maxs = [];
      series.forEach(function (serie) {
        var data = serie.data;
        for (var i = 0; i < data.length; i++) {
          var value = data[i];
          if (i >= maxs.length) {
            maxs.push(value);
          } else {
            maxs[i] = maxs[i] + value;
          }
        }
      });
      var max = _d2.default.max(maxs) || 0;
      return [0, max * 1.05];
    }
  }]);

  function _class(config) {
    (0, _classCallCheck3.default)(this, _class);
    var container = config.container;
    var category = config.category;
    var cols = config.cols;
    var data = config.data;

    //  画布周边的空白

    var padding = {
      left: 45,
      right: 10,
      top: 50,
      bottom: 30
    };
    var color = new _color2.default('C2'); // 默认第二色系
    var width = container.clientWidth - padding.left - padding.right;
    var height = (container.clientHeight || width * .5) - padding.top - padding.bottom;

    config.width = width;
    config.height = height;
    config.padding = padding;

    /**
     * tooltip 浮层对象节点
     * @type {node}
     */
    var tooltipElement = _d2.default.select(container).append('div').attr('class', 'tooltip').style({
      position: 'absolute',
      left: 0,
      top: 0,
      opacity: 0,
      'pointer-events': 'none'
    });

    var arrow = _d2.default.select(container).append('div').attr('class', 'arrow-down').style({
      position: 'absolute',
      left: 0,
      top: 0,
      opacity: 0,
      'pointer-events': 'none'
    });

    /**
     * 转换数据格式
     */
    var series = (0, _keys2.default)(cols).map(function (key, i) {
      return {
        key: key,
        name: cols[key],
        data: [],
        color: color.getDefault(i) || color.getColor(0, color.getDefault())
      };
    });

    //  X轴节点数
    var categories = [];

    data.forEach(function (item) {
      categories.push(item[category]);
      series.forEach(function (serie) {
        serie.data.push(item[serie.key]);
      });
    });

    //  x轴的比例尺
    var xScale = _d2.default.scale.ordinal().domain(categories).rangeRoundBands([0, width]);

    //  y轴的比例尺
    var yScale = _d2.default.scale.linear().domain(this.getDomain(series)).range([height, 0]).nice();

    config.series = series;
    config.categories = categories;
    config.xScale = xScale;
    config.yScale = yScale;

    var svg = new _svg2.default(config).create({
      highBg: 1
    });

    var rect = new _stackedRect2.default({
      div: tooltipElement,
      svg: svg.getSvg(),
      arrow: arrow,
      height: height,
      padding: padding,
      cols: cols,
      xScale: xScale,
      yScale: yScale
    });

    series.map(function (serie, index) {
      rect.create(series, serie, categories, series.length, index);
    });

    var _this = this;
    var _series = [].concat((0, _toConsumableArray3.default)(series));
    new _legend2.default(config).create(_series, function () {
      //  onClickCallback
      series = _series.filter(function (serie) {
        return !serie.disabled;
      });

      yScale.domain(_this.getDomain(series)).nice();

      svg.setYScale(yScale);
      svg.setSeries(series);

      svg.getSvg().selectAll('rect.MyRect').remove();

      rect.resetYScale(yScale);

      series.map(function (serie, i) {
        rect.create(series, serie, categories, series.length, i);
      });
    });
  }

  return _class;
})(); /**
       * Copyright 2016-present, Baifendian, Inc.
       * All rights reserved.
       *
       * This source code is licensed under the BSD-style license found in the
       * LICENSE file in the root directory of this source tree. An additional grant
       * of patent rights can be found in the PATENTS file in the same directory.
       */

exports.default = _class;